@@ -1,32 +1,14 @@
-/****************************************************************************
-**
-** Copyright (C) 2007 Urs Wolfer <uwolfer @ kde.org>
-**
-** This file is part of KDE.
-**
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
-**
-** You should have received a copy of the GNU General Public License
-** along with this program; see the file COPYING. If not, write to
-** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-** Boston, MA 02110-1301, USA.
-**
-****************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2007 Urs Wolfer <uwolfer@kde.org>
+
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "bookmarkmanager.h"
 #include "mainwindow.h"
 #include "krdc_debug.h"
 
-#include <kbookmarks_version.h>
-#include <KBookmarks/KBookmarkOwner>
+#include <KBookmarkOwner>
 #include <KLocalizedString>
 
 #include <QStandardPaths>
@@ -43,12 +25,8 @@ BookmarkManager::BookmarkManager(KActionCollection *collection, QMenu *menu, Mai
     m_manager = KBookmarkManager::managerForFile(file, QLatin1String("krdc"));
     m_manager->setUpdate(true);
 
-#if KBOOKMARKS_VERSION < QT_VERSION_CHECK(5, 69, 0)
-    m_bookmarkMenu = new KBookmarkMenu(m_manager, this, menu, collection);
-#else
     m_bookmarkMenu = new KBookmarkMenu(m_manager, this, menu);
     collection->addActions(menu->actions());
-#endif
 
     KBookmarkGroup root = m_manager->root();
     KBookmark bm = root.first();
@@ -100,14 +78,14 @@ void BookmarkManager::addHistoryBookmark(RemoteView *view)
 
 void BookmarkManager::openBookmark(const KBookmark &bm, Qt::MouseButtons, Qt::KeyboardModifiers)
 {
-    emit openUrl(bm.url());
+    Q_EMIT openUrl(bm.url());
 }
 
 void BookmarkManager::openFolderinTabs(const KBookmarkGroup &bookmarkGroup)
 {
     KBookmark bm = bookmarkGroup.first();
     while (!bm.isNull()) {
-        emit openUrl(bm.url());
+        Q_EMIT openUrl(bm.url());
         bm = bookmarkGroup.next(bm);
     }
 }
@@ -171,7 +149,7 @@ QList<KBookmarkOwner::FutureBookmark> BookmarkManager::currentBookmarkList() con
 void BookmarkManager::addManualBookmark(const QUrl &url, const QString &text)
 {
     m_manager->root().addBookmark(text, url, QString());
-    emit m_manager->emitChanged();
+    Q_EMIT m_manager->emitChanged();
 }
 
 KBookmarkManager* BookmarkManager::getManager()
@@ -198,7 +176,8 @@ const QStringList BookmarkManager::findBookmarkAddresses(const KBookmarkGroup &g
 
 void BookmarkManager::removeByUrl(KBookmarkManager *manager, const QString &url, bool ignoreHistory, const QString updateTitle)
 {
-    foreach(const QString &address, findBookmarkAddresses(manager->root(), url)) {
+    const QStringList addresses = findBookmarkAddresses(manager->root(), url);
+    for (const QString &address : addresses) {
         KBookmark bm = manager->findByAddress(address);
         if (ignoreHistory && bm.parentGroup().metaDataItem(QLatin1String("krdc-history")) == QLatin1String("historyfolder")) {
             if (!updateTitle.isEmpty()) {
@@ -218,7 +197,8 @@ void BookmarkManager::removeByUrl(KBookmarkManager *manager, const QString &url,
 
 void BookmarkManager::updateTitle(KBookmarkManager *manager, const QString &url, const QString &title)
 {
-    foreach(const QString &address, findBookmarkAddresses(manager->root(), url)) {
+    const QStringList addresses = findBookmarkAddresses(manager->root(), url);
+    for (const QString &address : addresses) {
         KBookmark bm = manager->findByAddress(address);
         bm.setFullText(title);
         qCDebug(KRDC) << "Update" << bm.fullText();
