@@ -1,26 +1,10 @@
-/****************************************************************************
-**
-** Copyright (C) 2007 - 2013 Urs Wolfer <uwolfer @ kde.org>
-** Copyright (C) 2009 - 2010 Tony Murray <murraytony @ gmail.com>
-**
-** This file is part of KDE.
-**
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
-**
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
-**
-** You should have received a copy of the GNU General Public License
-** along with this program; see the file COPYING. If not, write to
-** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-** Boston, MA 02110-1301, USA.
-**
-****************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2007-2013 Urs Wolfer <uwolfer@kde.org>
+    SPDX-FileCopyrightText: 2009-2010 Tony Murray <murraytony@gmail.com>
+    SPDX-FileCopyrightText: 2021 Rafał Lalik <rafallalik@gmail.com>
+
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "mainwindow.h"
 #include "krdc_debug.h"
@@ -34,6 +18,7 @@
 #include "systemtrayicon.h"
 #include "tabbedviewwidget.h"
 #include "hostpreferences.h"
+#include "factorwidget.h"
 
 #include <KActionCollection>
 #include <KActionMenu>
@@ -45,7 +30,7 @@
 #include <KPluginMetaData>
 #include <KToggleAction>
 #include <KToggleFullScreenAction>
-#include <QDebug>
+#include <KToolBar>
 
 #include <QClipboard>
 #include <QDockWidget>
@@ -189,6 +174,10 @@ void MainWindow::setupActions()
     scaleAction->setIconText(i18n("Scale"));
     connect(scaleAction, SIGNAL(triggered(bool)), SLOT(scale(bool)));
 
+    FactorWidget * m_scaleSlider = new FactorWidget(i18n("Scaling Factor"), this, actionCollection());
+    QAction * scaleFactorAction = actionCollection()->addAction(QStringLiteral("scale_factor"), m_scaleSlider);
+    scaleFactorAction->setIcon(QIcon::fromTheme(QStringLiteral("configure")));
+
     KStandardAction::quit(this, SLOT(quit()), actionCollection());
     KStandardAction::preferences(this, SLOT(preferences()), actionCollection());
     QAction *configNotifyAction = KStandardAction::configureNotifications(this, SLOT(configureNotifications()), actionCollection());
@@ -276,7 +265,7 @@ void MainWindow::newConnection(const QUrl &newUrl, bool switchFullscreenWhenConn
     RemoteView *view = nullptr;
     KConfigGroup configGroup = Settings::self()->config()->group(QStringLiteral("hostpreferences")).group(url.toDisplayString(QUrl::StripTrailingSlash));
 
-    foreach(RemoteViewFactory *factory, m_remoteViewFactories) {
+    for (RemoteViewFactory *factory : qAsConst(m_remoteViewFactories)) {
         if (factory->supportsUrl(url)) {
             view = factory->createView(this, url, configGroup);
             qCDebug(KRDC) << "Found plugin to handle url (" << url.url() << "): " << view->metaObject()->className();
@@ -298,7 +287,11 @@ void MainWindow::newConnection(const QUrl &newUrl, bool switchFullscreenWhenConn
 
     view->showLocalCursor(prefs->showLocalCursor() ? RemoteView::CursorOn : RemoteView::CursorOff);
     view->setViewOnly(prefs->viewOnly());
-    if (! switchFullscreenWhenConnected) view->enableScaling(prefs->windowedScale());
+    bool scale_state = false;
+    if (switchFullscreenWhenConnected) scale_state = prefs->fullscreenScale();
+    else scale_state = prefs->windowedScale();
+
+    view->enableScaling(scale_state);
 
     connect(view, SIGNAL(framebufferSizeChanged(int,int)), this, SLOT(resizeTabWidget(int,int)));
     connect(view, SIGNAL(statusChanged(RemoteView::RemoteStatus)), this, SLOT(statusChanged(RemoteView::RemoteStatus)));
@@ -317,6 +310,10 @@ void MainWindow::newConnection(const QUrl &newUrl, bool switchFullscreenWhenConn
     tabChanged(newIndex); // force to update m_currentRemoteView (tabChanged is not emitted when start page has been disabled)
 
     view->start();
+    setFactor(view->hostPreferences()->scaleFactor());
+
+    Q_EMIT factorUpdated(view->hostPreferences()->scaleFactor());
+    Q_EMIT scaleUpdated(scale_state);
 }
 
 void MainWindow::openFromRemoteDesktopsModel(const QModelIndex &index)
@@ -326,8 +323,10 @@ void MainWindow::openFromRemoteDesktopsModel(const QModelIndex &index)
     if (!urlString.isEmpty()) {
         const QUrl url(urlString);
         // first check if url has already been opened; in case show the tab
-        foreach (QWidget *widget, m_remoteViewMap.keys()) {
-            if (m_remoteViewMap.value(widget)->url() == url) {
+        for (auto it = m_remoteViewMap.constBegin(), end = m_remoteViewMap.constEnd(); it != end; ++it) {
+            RemoteView *view = it.value();
+            if (view->url() == url) {
+                QWidget *widget = it.key();
                 m_tabWidget->setCurrentWidget(widget);
                 return;
             }
@@ -456,6 +455,9 @@ void MainWindow::switchFullscreen()
 {
     qCDebug(KRDC);
 
+    RemoteView* view = currentRemoteView();
+    bool scale_state = false;
+
     if (m_fullscreenWindow) {
         // Leaving full screen mode
         m_fullscreenWindow->setWindowState(Qt::WindowNoState);
@@ -469,7 +471,7 @@ void MainWindow::switchFullscreen()
         restoreGeometry(m_mainWindowGeometry);
         if (m_systemTrayIcon) m_systemTrayIcon->setAssociatedWidget(this);
 
-        foreach (RemoteView * view, m_remoteViewMap) {
+        for (RemoteView * view : qAsConst(m_remoteViewMap)) {
             view->enableScaling(view->hostPreferences()->windowedScale());
         }
 
@@ -482,6 +484,8 @@ void MainWindow::switchFullscreen()
         actionCollection()->action(QStringLiteral("switch_fullscreen"))->setIcon(QIcon::fromTheme(QStringLiteral("view-fullscreen")));
         actionCollection()->action(QStringLiteral("switch_fullscreen"))->setText(i18n("Switch to Full Screen Mode"));
         actionCollection()->action(QStringLiteral("switch_fullscreen"))->setIconText(i18n("Full Screen"));
+        if (view)
+            scale_state = view->hostPreferences()->windowedScale();
 
         m_fullscreenWindow->deleteLater();
         m_fullscreenWindow = nullptr;
@@ -496,7 +500,7 @@ void MainWindow::switchFullscreen()
         m_tabWidget->tabBar()->hide();
         m_tabWidget->setDocumentMode(true);
 
-        foreach(RemoteView *currentView, m_remoteViewMap) {
+        for (RemoteView *currentView : qAsConst(m_remoteViewMap)) {
             currentView->enableScaling(currentView->hostPreferences()->fullscreenScale());
         }
 
@@ -520,10 +524,18 @@ void MainWindow::switchFullscreen()
         actionCollection()->action(QStringLiteral("switch_fullscreen"))->setText(i18n("Switch to Window Mode"));
         actionCollection()->action(QStringLiteral("switch_fullscreen"))->setIconText(i18n("Window Mode"));
         showRemoteViewToolbar();
+        if (view)
+            scale_state = view->hostPreferences()->fullscreenScale();
     }
     if (m_tabWidget->currentWidget() == m_newConnectionWidget && m_addressInput) {
         m_addressInput->setFocus();
     }
+
+    if (view) {
+        Q_EMIT factorUpdated(view->hostPreferences()->scaleFactor());
+        Q_EMIT scaleUpdated(scale_state);
+    }
+    actionCollection()->action(QStringLiteral("scale"))->setChecked(scale_state);
 }
 
 QScrollArea *MainWindow::createScrollArea(QWidget *parent, RemoteView *remoteView)
@@ -626,7 +638,7 @@ void MainWindow::showSettingsDialog(const QString &url)
 {
     HostPreferences *prefs = nullptr;
 
-    foreach(RemoteViewFactory *factory, remoteViewFactoriesList()) {
+    for (RemoteViewFactory *factory : qAsConst(m_remoteViewFactories)) {
         if (factory->supportsUrl(QUrl(url))) {
             prefs = factory->createHostPreferences(Settings::self()->config()->group(QStringLiteral("hostpreferences")).group(url), this);
             if (prefs) {
@@ -755,6 +767,13 @@ void MainWindow::grabAllKeys(bool grabAllKeys)
     saveHostPrefs(view);
 }
 
+void setActionStatus(QAction* action, bool enabled, bool visible, bool checked)
+{
+    action->setEnabled(enabled);
+    action->setVisible(visible);
+    action->setChecked(checked);
+}
+
 void MainWindow::scale(bool scale)
 {
     qCDebug(KRDC);
@@ -767,6 +786,23 @@ void MainWindow::scale(bool scale)
         view->hostPreferences()->setWindowedScale(scale);
 
     saveHostPrefs(view);
+
+    Q_EMIT scaleUpdated(scale);
+}
+
+void MainWindow::setFactor(int scale)
+{
+    float s = float(scale)/100.;
+
+    RemoteView* view = currentRemoteView();
+    if (view) {
+        view->setScaleFactor(s);
+
+        view->enableScaling(view->scaling());
+        view->hostPreferences()->setScaleFactor(scale);
+
+        saveHostPrefs(view);
+    }
 }
 
 void MainWindow::showRemoteViewToolbar()
@@ -803,6 +839,7 @@ void MainWindow::showRemoteViewToolbar()
         buttonBox->addAction(actionCollection()->action(QStringLiteral("show_local_cursor")));
         buttonBox->addAction(actionCollection()->action(QStringLiteral("grab_all_keys")));
         buttonBox->addAction(actionCollection()->action(QStringLiteral("scale")));
+        buttonBox->addAction(actionCollection()->action(QStringLiteral("scale_factor")));
         buttonBox->addAction(actionCollection()->action(QStringLiteral("disconnect")));
         buttonBox->addAction(actionCollection()->action(QStringLiteral("file_quit")));
 
@@ -817,13 +854,6 @@ void MainWindow::showRemoteViewToolbar()
     }
 }
 
-void setActionStatus(QAction* action, bool enabled, bool visible, bool checked)
-{
-    action->setEnabled(enabled);
-    action->setVisible(visible);
-    action->setChecked(checked);
-}
-
 void MainWindow::updateActionStatus()
 {
     qCDebug(KRDC) << m_tabWidget->currentIndex();
@@ -853,6 +883,9 @@ void MainWindow::updateActionStatus()
                     view ? view->supportsScaling() : false,
                     view ? view->scaling() : false);
 
+    actionCollection()->action(QStringLiteral("scale_factor"))->setEnabled(enabled);
+    actionCollection()->action(QStringLiteral("scale_factor"))->setVisible(view ? view->supportsScaling() : false);
+
     setActionStatus(actionCollection()->action(QStringLiteral("grab_all_keys")),
                     enabled,
                     enabled,
@@ -914,7 +947,7 @@ void MainWindow::updateConfiguration()
     }
 
     // Send update configuration message to all views
-    foreach (RemoteView *view, m_remoteViewMap) {
+    for (RemoteView *view : qAsConst(m_remoteViewMap)) {
         view->updateConfiguration();
     }
 }
@@ -930,7 +963,7 @@ void MainWindow::quit(bool systemEvent)
 
         if (Settings::rememberSessions()) { // remember open remote views for next startup
             QStringList list;
-            foreach (RemoteView *view, m_remoteViewMap) {
+            for (RemoteView *view : qAsConst(m_remoteViewMap)) {
                 qCDebug(KRDC) << view->url();
                 list.append(view->url().toDisplayString(QUrl::StripTrailingSlash));
             }
@@ -939,7 +972,8 @@ void MainWindow::quit(bool systemEvent)
 
         saveHostPrefs();
 
-        foreach (RemoteView *view, m_remoteViewMap) {
+        const QMap<QWidget *, RemoteView *> currentViews = m_remoteViewMap;
+        for (RemoteView *view : currentViews) {
             view->startQuitting();
         }
 
@@ -996,7 +1030,7 @@ void MainWindow::saveProperties(KConfigGroup &group)
 
 void MainWindow::saveHostPrefs()
 {
-    foreach (RemoteView *view, m_remoteViewMap) {
+    for (RemoteView *view : qAsConst(m_remoteViewMap)) {
         saveHostPrefs(view);
     }
 }
@@ -1058,7 +1092,7 @@ QWidget* MainWindow::newConnectionWidget()
         m_addressInput->setPlaceholderText(i18n("Type here to connect to an address and filter the list."));
         connect(m_addressInput, SIGNAL(textChanged(QString)), remoteDesktopsModelProxy, SLOT(setFilterFixedString(QString)));
 
-        foreach(RemoteViewFactory *factory, m_remoteViewFactories) {
+        for (RemoteViewFactory *factory : qAsConst(m_remoteViewFactories)) {
             m_protocolInput->addItem(factory->scheme());
         }
         m_protocolInput->setCurrentText(Settings::defaultProtocol());
@@ -1163,7 +1197,7 @@ void MainWindow::createDockWidget()
     remoteDesktopsDockWidget->setObjectName(QStringLiteral("remoteDesktopsDockWidget")); // required for saving position / state
     remoteDesktopsDockWidget->setWindowTitle(i18n("Remote Desktops"));
     QFontMetrics fontMetrics(remoteDesktopsDockWidget->font());
-    remoteDesktopsDockWidget->setMinimumWidth(fontMetrics.width(QStringLiteral("vnc://192.168.100.100:6000")));
+    remoteDesktopsDockWidget->setMinimumWidth(fontMetrics.horizontalAdvance(QStringLiteral("vnc://192.168.100.100:6000")));
     actionCollection()->addAction(QStringLiteral("remote_desktop_dockwidget"),
                                   remoteDesktopsDockWidget->toggleViewAction());
 
